/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 8.8.2018
 * Time: 16:52
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 *
 * @param {AntivirusInfo} avInfo must be valid instance of {@link AntivirusInfo}
 * @param {Boolean} enabled enable AV scans as general
 * @param {String} device - This is the tag used for a specific install and defines a category for a list of
 *                        antivirus software from the AV list file. It is a combination of "platform" and
 *                        "operating system", and will look something like Linux-Synology or Windows-PC. This value is
 *                        defaulted to the platform-OS pair but can be overridden by an installing to use a custom tag if required.
 * @param {Boolean} state True if the AV command tested successfully. False if no AV (including the default) were tested successfully.

 * @constructor
 */
var AntivirusConfig = function (avInfo, enabled, device, state) {
    if (false === avInfo instanceof AntivirusInfo)
        throw new Error("Invalid avInfo: " + avInfo);

    /**
     *
     * @type {AntivirusInfo}
     * @private
     */
    this._avInfo = avInfo;

    /**
     * Enabled enable AV scans (for now only on downloads, e.g. ).
     *
     * @type {Boolean}
     * @private
     */
    this._enabled = enabled;

    /**
     *
     * @type {String}
     * @private
     */
    this._device = device;

    /**
     *
     * @type {Boolean}
     * @private
     */
    this._state = !!state;
};

QnUtil.extend(AntivirusConfig.prototype, {

    /* Public method */

    /**
     * Check did current configuration is proper based on status flag returned in status
     *
     * @return {Boolean} always valid
     */
    isConfigProper: function () {
        return this._state;
    },

    /**
     * Get device that is used to make request for getting list of antivirus that thi device support
     *
     * @return {String} always valid non empty string
     */
    getDevice: function () {
        var device = this._device;
        // fallback if device is not found in options will crete it from platform + "-" + os
        // This should be happened only when config is created on init (default value)
        // before response of status request is received
        if (!device) {
            var platform = getGlobal(KEY_GLOBAL_DEVICE_OS);
            platform = platform.substring(0, 1).toLocaleUpperCase() + platform.substring(1, platform.length);
            device = platform + "-" + getGlobal(KEY_GLOBAL_DEVICE_SCHEME).toLocaleUpperCase();
        }

        return device;
    },

    /**
     * Get antivirus id
     *
     * @return {String} always valid non empty string
     */
    getAvid: function () {
        return this._avInfo.getAvid();
    },

    /**
     * @returns {boolean} true if the antivirus was not set by the user yet and default behaviour
     * should be presented.
     */
    isNone: function () {
        return !this._enabled && this._avInfo.isNone();
    },

    /**
     * @returns {boolean} true if the antivirus was not set by the user yet and default behaviour
     * should be presented.
     */
    isNotSet: function () {
        return this._enabled && this._avInfo.isNotSet();
    },

    /**
     * Get Antivirus Info
     *
     * @return {AntivirusInfo}
     */
    getAvInfo: function () {
        return this._avInfo;
    },

    /**
     * Clone current config with given antivirus info {@link AntivirusInfo}.
     * Enabled flag will be set to true in all cases except if {@link AntivirusInfo#isNone}=true then set to false!
     *
     * @param {AntivirusInfo} avInfo must be valid.
     * @return {AntivirusConfig} new clone instance
     */
    cloneWithAvInfo: function (avInfo) {
        return new AntivirusConfig(avInfo, !avInfo.isNone(), this._device, this._state);
    },

    /**
     * Result format is:
     * {
     *      "upload": false,
     *      "download": true,
     *      "builtin_enabled": false,
     *      "builtin_fresh_hours: 24,
     *      "device": "Windows-PC",
     *      "name": "Antivirus Name",
     *      "avcmd": "some/full/file/cmd2/software.exe"
     *      "maxsize": 0,
     *      "filter": "",
     *      "enabled": false
     * }
     *
     * @param {Object} root
     */
    toProto: function (root) {
        root["enabled"] = this._enabled;
        root["download"] = true;
        root["upload"] = false;
        root["device"] = this._device;
        // No build-in for now
        root["builtin_enabled"] = false;
        root["builtin_fresh_hours"] = 0;

        this._avInfo.toProto(root);
    },

    toString: function () {
        return "Enabled=" + this._enabled +
            ", device:" + this._device +
            ", status:" + this._state +
            ", AVInfo: " + this._avInfo;
    }

    /* Protected method */
    /* Private method */

});

/**
 * Format is:
 * {
 *      "upload": false,
 *      "download": true,
 *      "builtin": false,
 *      "builtin_enabled": false,
 *      "builtin_fresh_hours": 24,
 *      "builtin_cmd": "",
 *      "builtin_fresh_cmd": "",
 *      "cmd": "",
 *      "name": "",
 *      "device": "Windows-PC",
 *      "filter": "",
 *      "maxsize": 0,
 *      "state": true,
 *      "enabled": false
 * }
 * Where:
 * {Boolean} download - enable scans on download. Default=true
 * {Boolean} upload - enable scans on upload (passive sending a file). Default=false
 * {Boolean} builtin_enabled - Is the built_in scanner being used? Should be false since we don't use one.
 * {Number} builtin_fresh_hours - How often will the built_in scan be executed? Should be 0 since we don't use one.
 *                                    Was expected to be 24 hours if we did use one.
 *
 * @param {Object} root
 * @return {AntivirusConfig}
 */
AntivirusConfig.fromProto = function (root) {
    var avInfo;
    var enabled = QnUtil.getBoolOpt(root, "enabled", false);
    try {
        if(enabled) {
            var name = QnUtil.getStr(root, "name");
            // Empty indicates no AV
            if (!name) {
                avInfo = AntivirusInfo.NOT_SET;
            } else {
                avInfo = AntivirusInfo.fromProto(root);
            }
        } else {
            avInfo = AntivirusInfo.NONE;
        }
    } catch (e) {
        logWarn("Invalid selected AV info: " + JSON.stringify(root), e);
        avInfo = AntivirusInfo.NOT_SET;
    }
    var upload = QnUtil.getBoolOpt(root, "upload", false);
    if (upload)
        logWarn("AV scan on upload is ON");
    var device = QnUtil.getStrOpt(root, "device", null);
    var state = QnUtil.getBoolOpt(root, "state", true);

    return new AntivirusConfig(avInfo, enabled, device, state);
};

/**
 * Represents not selected AV.
 *
 * @type {AntivirusConfig}
 * @constant
 */
AntivirusConfig.NOT_SET = new AntivirusConfig(AntivirusInfo.NOT_SET, true, null, true);
