/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 27.7.2018
 * Time: 10:52
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 *
 * @param {String} avid unique antivirus id
 * @param {String} name display name
 * @param {String} cmd The command to execute the user select antivirus during upload or downloads
 *                         (dependent on settings). Must be valid. Can be empty string only in case when avid is
 *                         {@link AntivirusInfo#NONE_ID} or {@link AntivirusInfo#CUSTOM_ID}
 * @param  {Number} maxSize - maximum size of file that can be scanned.
 * Any file larger than this will fail during the scan and potentially be deleted.
 * This should only be used when an antivirus is incapable
 *                           of scanning files above a particular size.
 * @param {String} filter list of comma separated extensions that will be ignored by the
 *                          antivirus scanner (ex. "mp3,np4"). Default is empty string
 * @param {Boolean} isDefault This is the default antivirus for a given system.
 *
 * @constructor
 */
var AntivirusInfo = function (avid, name, cmd, maxSize, filter, isDefault) {
    if (!QnUtil.isStr(avid) || !avid)
        throw new Error("Invalid AV id: " + avid);
    if (!QnUtil.isStr(name) || !name)
        throw new Error("Invalid AV name: " + name);
    if (!QnUtil.isStr(cmd))
        throw new Error("Invalid cmd: " + cmd);

    switch (avid) {
        case AntivirusInfo.NONE_ID:
        case AntivirusInfo.NOT_SET_ID:
            if (cmd !== "")
                throw new Error("Invalid " + avid + " cmd: " + cmd);
            break;
        case AntivirusInfo.CUSTOM_ID:
            break;
        default:
            if (!cmd)
                throw new Error("Empty cmd: " + cmd);
            break;
    }

    /**
     *
     * @type {String}
     * @private
     */
    this._avid = avid;
    /**
     *
     * @type {String}
     * @private
     */
    this._name = name;
    /**
     *
     * @type {String}
     * @private
     */
    this._cmd = cmd;
    /**
     *
     * @type {Number}
     * @private
     */
    this._maxSize = maxSize;
    /**
     *
     * @type {String}
     * @private
     */
    this._filter = filter;
    /**
     *
     * @type {Boolean}
     * @private
     */
    this._isDefault = isDefault;
};

QnUtil.extend(AntivirusInfo.prototype, {

    /* Public method */

    /**
     * Get antivirus id
     *
     * @return {String} always valid non empty string
     */
    getAvid: function () {
        return this._avid;
    },

    /**
     *
     * @returns {boolean} true if this AV is {@link AntivirusInfo#NONE},
     * e.g. it has ID={@link AntivirusInfo#NONE_ID}.
     */
    isNone: function () {
        return this._avid === AntivirusInfo.NONE_ID;
    },

    /**
     *
     * @returns {boolean} true if this AV is {@link AntivirusInfo#NOT_SET},
     * e.g. it has ID={@link AntivirusInfo#NOT_SET_ID}.
     */
    isNotSet: function () {
        return this._avid === AntivirusInfo.NOT_SET_ID;
    },

    /**
     * @returns {boolean} true if this is the default antivirus for a given system.
     */
    isDefault: function () {
        return this._isDefault;
    },

    /**
     * Get antivirus display name
     *
     * @return {String} always valid non empty string
     */
    getAvName: function () {
        return this._name;
    },

    /**
     * Get antivirus cmd
     *
     * @return {String} always valid but can be empty string in some case
     */
    getAvCmd: function () {
        return this._cmd;
    },

    /**
     * Result format is:
     * {
     *      "name": "Antivirus Name",
     *      "avcmd": "some/full/file/cmd2/software.exe"
     *      "maxsize": 0,
     *      "filter": ""
     * }
     *
     * @param {Object} root
     */
    toProto: function (root) {
        root["name"] = this._avid !== AntivirusInfo.NONE_ID ? this._name : "";
        root["avcmd"] = this._cmd;
        root["maxsize"] = this._maxSize;
        root["filter"] = this._filter;
    },

    /**
     *
     * @param {String} cmd
     * @return {AntivirusInfo}
     */
    cloneWithCmd: function (cmd) {
        return new AntivirusInfo(this._avid, this._name, cmd, this._maxSize, this._filter, this._isDefault);
    },

    toString: function () {
        return "avid: " + this._avid + ", name:" + this._name + ", default=" + this._isDefault +
            ", cmd='" + this._cmd + "', maxSize=" + this._maxSize + ", filter=" + this._filter;
    }

    /* Protected method */
    /* Private method */

});

/**
 * Parse antivirus from status info
 * Format is:
 * {
 *      "name": "Antivirus Name",
 *      "cmd": "some/full/file/cmd2/software.exe"
 *      "default": true,
 *      "maxfilesize": 0,
 *      "filter": ""
 * }
 *
 * @param {Object} root raw antivirus object
 * @return {AntivirusInfo}
 */
AntivirusInfo.fromProto = function (root) {
    var name = QnUtil.getStr(root, "name");
    // Petar: for now just set some avid in most case name or some of predefined
    //var avid = root["avid"];
    var avid = name.toLocaleLowerCase() === AntivirusInfo.CUSTOM_ID ? AntivirusInfo.CUSTOM_ID : name;
    var cmd = QnUtil.getStr(root, "cmd");
    // Try with both
    var maxSize = QnUtil.getNumberOpt(root, "maxfilesize", null);
    if (!maxSize)
        maxSize = QnUtil.getNumberOpt(root, "maxsize", 0);
    var filter = QnUtil.getStrOpt(root, "filter", "");
    var isDefault = QnUtil.getBoolOpt(root, "default", false);

    return new AntivirusInfo(avid, name, cmd, maxSize, filter, isDefault);
};

/**
 * Parse antivirus from status info list with detected
 * Format is:
 * [
 * {
 *      "name": "Antivirus Name",
 *      ...
 * },
 *   ...
 * ]
 * For format of one AV, see {@link AntivirusInfo#fromProto}.
 *
 * @param {Array} list
 * @return {AntivirusInfo[]}
 */
AntivirusInfo.fromList = function (list) {
    var tmp = [];
    for (var i = 0; i < list.length; i++) {
        var cur = list[i];
        try {
            var av = AntivirusInfo.fromProto(cur);
            if (av._avid === AntivirusInfo.NONE_ID || av._avid === AntivirusInfo.NOT_SET_ID) {
                logWarn("Cannot use predefined AV: " + av);
            } else {
                tmp.push(av);
                if (isLogDbg())
                    logDbg("Parse AV: " + av);
            }
        } catch (e) {
            logWarn("Invalid AV info: " + JSON.stringify(cur), e);
        }
    }

    return tmp;
};

/**
 *
 * @type {string}
 * @constant
 */
AntivirusInfo.NOT_SET_ID = "notset";
/**
 *
 * @type {string}
 * @constant
 */
AntivirusInfo.NONE_ID = "none";
/**
 *
 * @type {string}
 * @constant
 */
AntivirusInfo.CUSTOM_ID = "custom";

/**
 *
 * @type {AntivirusInfo}
 * @constant
 */
AntivirusInfo.NOT_SET = new AntivirusInfo(AntivirusInfo.NOT_SET_ID, "NotSet", "", 0, "", false);
/**
 *
 * @type {AntivirusInfo}
 * @constant
 */
AntivirusInfo.NONE = new AntivirusInfo(AntivirusInfo.NONE_ID, "None", "", 0, "", false);
/**
 *
 * @type {AntivirusInfo}
 * @constant
 */
AntivirusInfo.CUSTOM = new AntivirusInfo(AntivirusInfo.CUSTOM_ID, "CUSTOM", "", 0, "", false);
