/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 24.7.2018
 * Time: 17:44
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 *
 * @param {jQuery} $container
 * @param {Object} options
 * @param {Boolean} doNotRequestStatus
 * @constructor
 */
var FrameAntivirusSettings = function ($container, options, doNotRequestStatus) {
    /**
     *
     * @type {CompAntivirusSettings}
     * @private
     */
    this._compAvsSettings = null;

    FrameWithForm.call(this, FrameAntivirusSettings.VID, $container, options, doNotRequestStatus);

};

QnUtil.inherits(FrameAntivirusSettings, FrameWithForm);
QnUtil.extend(FrameAntivirusSettings.prototype, {

    /* Public method */

    updateFrame: function(isFromStatus) {
        if(isFromStatus)
            this._getAvList();
        // Hide "Cancel" buttons for browsers
        this._toggleBtn(BTN_CANCEL_ID);

        this._compAvsSettings.updatePanel();

        FrameAntivirusSettings.superclass.updateFrame.call(this, isFromStatus);

        this._updateSubmitBtnState();
    },


    /* Protected method */

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getBtnIdList: function () {
        return [BTN_SAVE_ID, BTN_CANCEL_ID];
    },

    /**
     *
     * @return {string}
     * @protected
     */
    _getControlIdPrefix: function () {
        return "avs-";
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _renderPageTitle: function ($cont) {
        var $desc = $('<h2 class="secureonly"></h2>')
            .setTextById("antivirus_settings_title");
        $desc.appendTo($cont);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _renderFormImpl: function ($container) {
        this._compAvsSettings = new CompAntivirusSettings($container);

        var self = this;
        $container.on("change", $('select'),function (e) {
            self._handleGuiEvent($(this), e);
        });

        $container.on("keyup", $('input.filed_avs_command_line'),function (e) {
            self._handleGuiEvent($(this), e);
        });
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitImpl: function (event) {
        var selected = this._compAvsSettings ? this._compAvsSettings.getSelectedAv() : null;
        var hasChange = !!selected;
        if(hasChange) {
            // We need to add new lines before and after AV name
            // But do not add them strings because translation system will not work
            hasChange = confirm(QnLocale.getStr("antivirus_settings_save_confirm",
                {listArg: {"avName": "\n" + selected.getAvName() + "\n"}, noEsc: true}));
        }
        return hasChange;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitSuccessImpl: function (resp, requestParams) {
        if (resp.result) {
            var redirectURL = resp["redirectURL"];
            log("Antivirus settings update successful");
            // indicate success visually
            showMessage(ERROR_OK, "antivirus_settings_success", this._getMainCont().find("#updatemsgForm"));

            // if a redirect URL was provided by passive (via the server) we use it to send the user back immediately
            if (redirectURL != null) {
                // if there is a redirect URL, we immediately redirect the user
                log("redirectURL found: " + redirectURL);
                window.location.href = redirectURL;
            } else {
                log("no redirect URL was found");
            }

        } else {
            log("Antivirus settings update failure");
        }
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitFailImpl: function (xhr, err) {
        if (xhr.responseText) {
            try {
                var response = JSON.parse(xhr.responseText);
                var error = response["error"];
                log("Antivirus settings update failed error code: " + error["code"]);
            } catch (e) {
                log("Antivirus settings update failed: " + e);
            }
        }
    },

    /**
     * Enable/Disable form controls (inputs, buttons etc..)
     *
     * @param {Boolean} [onDisable]
     * @param {Boolean} [notForSubmit]
     * @protected
     */
    _setControlsState: function (onDisable, notForSubmit) {
        // update input
        this._compAvsSettings.setControlsState(onDisable);
        // update buttons
        var $btnSubmit = this._getBtn(BTN_SAVE_ID);
        if(!notForSubmit)
            $btnSubmit.button(onDisable ? 'loading' : 'reset');
        $btnSubmit.attr("disabled", onDisable);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _fillSubmitParams: function (params) {
        params["cmd"] = "set-antivirus";
        var avConfig = getGlobal(KEY_GLOBAL_SELECTED_ANTIVIRUS);
        var selected = this._compAvsSettings.getSelectedAv();
        var /*AntivirusConfig*/newConfig = null !== selected ? avConfig.cloneWithAvInfo(selected) : avConfig;
        if (isLogInfo())
            logInfo("New AV config: " + newConfig + ". From: " + avConfig);
        newConfig.toProto(params);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getRequestUrl: function () {
        return "/secure/";
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleGuiEvent: function ($comp, event) {
        this._updateSubmitBtnState();
    },


    /* Private method */

    _getAvList: function () {
        var self = this;
        var device = getGlobal(KEY_GLOBAL_SELECTED_ANTIVIRUS).getDevice();
        var data = {
            "cmd": "get-antivirus-list",
            "device": device
        };

        var jqXHR = $.ajax({
            method: "GET",
            url: this._getRequestUrl(),
            data: data,
            dataType: "json"
        });

        // noinspection JSUnresolvedFunction
        var code;
        jqXHR.done(function (resp) {
            log("fetch av list success with: ");
            log(resp);
            if(!resp["entries"]) {
                code = resp.code;
                var respMsg = resp.msg;
                if(!code && respMsg)
                    code = translateMsgToErrorCode(respMsg);
                translateErrorCode(code, self._getMainCont().find("#updatemsgForm"));
            } else {
                // parse list with antivirus
                var avList = AntivirusInfo.fromList(resp["entries"]);
                avList.push(AntivirusInfo.CUSTOM);
                avList.push(AntivirusInfo.NONE);

                self._compAvsSettings.updatePanel(avList);
            }
        }).fail(function (jqXHR, textStatus, ex) {
            logWarn("fetch av list failure status: " + jqXHR.status + " " + textStatus +
                (jqXHR.responseText ? ", response: " + jqXHR.responseText : "") +
                (ex ? ", exception: " + ex : ""));
        });
    },

    /**
     * Enable/Disable save button
     *
     * @private
     */
    _updateSubmitBtnState: function () {
        var selected = this._compAvsSettings.getSelectedAv();
        var isDisabled = null === selected || selected.isNotSet();
        // Comment because need to show None options in case of MSP
        // See https://jira2.cnexus.com/browse/FF-3782
        // || (selected.isNone() && getGlobal(KEY_GLOBAL_AV_OBLIGATORY));

        this._getBtn(BTN_SAVE_ID).prop("disabled", isDisabled);
    }

});

/**
 *
 * @type {string}
 * @constant
 */
FrameAntivirusSettings.VID = "FrameAntivirusSettings";
